/*
** Module   : Buffer.java
** Abstract : interface which must be extended by any business DMO interface
**
** Copyright (c) 2013, Golden Code Development Corporation.
** ALL RIGHTS RESERVED. Use is subject to license terms.
**
**           Golden Code Development Corporation
**                      CONFIDENTIAL
**
** -#- -I- --Date-- ---------------------------------Description---------------------------------
** 001 ECF 20130118 Created initial version.
** 002 VMN 20130207 Added support for handle-based methods:
**                  BUFFER-COPY (KW_BUF_COPY)
**                  BUFFER-COMPARE (KW_BUF_COMP)
**                  BUFFER-FIELD (KW_BUF_FLD)
**                  BUFFER-RELEASE (KW_BUF_REL)
**                  FIND-BY-ROWID (KW_FIND_BR)
**                  FIND-UNIQUE (KW_GET_UNI)
**                  FIND-FIRST (KW_FIND_1ST)
**                  FIND-LAST (KW_FIND_LST)
**                  and attributes:
**                  NUM-FIELDS (KW_NUM_FLD).
**                  INITIAL (KW_INIT)
**                  AMBIGUOUS (KW_AMBIG)
*/

package com.goldencode.p2j.persist;

import com.goldencode.p2j.util.*;

/**
 * Defines the public API for services representing Progress builtin
 * functions, methods, attributes, and language statements, which operate on
 * record buffers.
 * <p>
 * Each DMO interface should contain a public, inner interface which extends
 * both this class and the enclosing DMO interface. This enables DMO proxies
 * used by business logic to invoke DMO-specific methods, as well as methods
 * which provide standard buffer services, without requiring a cast.
 */
public interface Buffer
extends WrappedResource
{
   /**
    * Get the record ID of the current buffer's current record, if any.  P2J
    * implementation of the RECID function.
    * <p>
    * This method will return a 32-bit recid which represents the internal
    * row ID of the given record.  If the actual row ID (which is implemented
    * as a wider integer) cannot fit due to overflow, an error condition will
    * be raised.
    *
    * @return  Record ID of the underlying data record. If no record currently
    *          backs the buffer associated with <code>dmo</code>, a recid
    *          initialized as unknown value is returned.
    * 
    * @throws  ErrorConditionException
    *          if the actual record ID is too wide to fit within a 32-bit
    *          integer.
    */
   public recid recordID();
   
   /**
    * Get the row ID of the current buffer's current record, if any.  P2J
    * implementation of the ROWID function.
    *
    * @return  Row ID of the underlying data record.  If no record currently
    *          backs the buffer associated with <code>dmo</code>, a
    *          <code>rowid</code> initialized as unknown value is returned.
    */
   public rowid rowID();
   
   /**
    * Indicate whether a backing data record currently is available.
    *
    * @return  <code>true</code> if data is available, <code>false</code> if
    *          no record backs this buffer.
    */
   public logical available();
   
   /**
    * Indicate whether a backing data record currently is available.
    *
    * @return  <code>true</code> if data is available, <code>false</code> if
    *          no record backs this buffer.
    */
   public boolean _available();
   
   /**
    * Checks if the record in the buffer has changed in database since the 
    * last <code>FIND CURRENT</code> or <code>GET CURRENT</code>.
    * P2J implementation of 4GL function <code>CURRENT-CHANGED</code>.
    *
    * @return  <code>true</code> if the database record differs from the
    *          value in memory buffer that was queried with the last
    *          <code>FIND</code> and <code>false</code> otherwise
    */
   public logical currentChanged();
   
   /**
    * Checks if the record in the buffer has changed in database since the 
    * last <code>FIND CURRENT</code> or <code>GET CURRENT</code>.
    * P2J implementation of 4GL function <code>CURRENT-CHANGED</code>.
    *
    * @return  <code>true</code> if the database record differs from the
    *          value in memory buffer that was queried with the last
    *          <code>FIND</code> and <code>false</code> otherwise
    */
   public boolean _currentChanged();
   
   /**
    * Indicate whether the buffer associated currently contains a record, and
    * whether that record is transient.  A record is transient if it has been
    * newly created and has not yet been saved to the database.
    *
    * @return  <code>true</code> if a record exists and is new, else
    *          <code>false</code>.
    */
   public logical newlyCreated();
   
   /**
    * Indicate whether the buffer associated currently contains a record, and
    * whether that record is transient.  A record is transient if it has been
    * newly created and has not yet been saved to the database.
    *
    * @return  <code>true</code> if a record exists and is new, else
    *          <code>false</code>.
    */
   public boolean _newlyCreated();
   
   /**
    * Indicate whether the last record retrieval attempt was denied because
    * the target record was locked.
    *
    * @return  <code>true</code> if the record was locked;  <code>false</code>
    *          if the attempt succeeded, or if this query has not yet
    *          attempted to retrieve a record.
    */
   public logical locked();
   
   /**
    * Indicate whether the last record retrieval attempt was denied because
    * the target record was locked.
    *
    * @return  <code>true</code> if the record was locked;  <code>false</code>
    *          if the attempt succeeded, or if this query has not yet
    *          attempted to retrieve a record.
    */
   public boolean _locked();
   
   /**
    * Indicate whether the last record retrieval attempt failed because the
    * query criteria was ambiguous.
    *
    * @return  <code>true</code> if the query was ambiguous;
    *          <code>false</code> if the attempt succeeded, or if no attempt
    *          has yet been made.
    */
   public logical ambiguous();
   
   /**
    * Indicate whether the last record retrieval attempt failed because the
    * query criteria was ambiguous.
    *
    * @return  <code>true</code> if the query was ambiguous;
    *          <code>false</code> if the attempt succeeded, or if no attempt
    *          has yet been made.
    */
   public boolean _ambiguous();
   
   /**
    * Create a new record in the buffer.
    *
    * @throws  ErrorConditionException
    *          if there is an error retrieving the next available primary key
    *          from the database or locking the record.
    */
   public void create();
   
   /**
    * Delete the record currently in the buffer, if any.
    *
    * @throws  ErrorConditionException
    *          if no record currently is loaded in the buffer, or if there is
    *          an error deleting the current record from the database.
    */
   public void delete();
   
   /**
    * Delete all records associated with this buffer, which match the criteria
    * specified by the given where clause.  Note that this where clause must
    * not contain DMO alias qualifiers.
    * <p>
    * <b>NOTE:</b>  the current implementation must not be invoked for a DMO
    * which is backed by a permanent table;  only temp-table bulk delete is
    * supported at this time.
    * 
    * @param   where
    *          An HQL where clause snippet which defines the restriction
    *          criteria to apply to the delete.  All references to properties
    *          in a DMO must be unqualified.
    * @param   args
    *          Query substitution parameters required by the where clause.
    * 
    * @throws  ErrorConditionException
    *          if there is an error deleting the records.
    * @throws  UnsupportedOperationException
    *          if invoked on a buffer associated with a permanent table.
    */
   public void delete(String where, Object ...args);
   
   /**
    * Bulk delete all records associated with the buffer.
    * 
    * @throws  ErrorConditionException
    *          if an error occurs performing the bulk delete operation.
    */
   public void deleteAll();
   
   /**
    * Validate and release the record currently in the buffer, if any.
    *
    * @throws  ErrorConditionException
    *          if the buffer fails validation.
    */
   public void release();
   
   /**
    * Validate the record currently in the buffer, if any.
    *
    * @throws  ErrorConditionException
    *          if the buffer fails validation.
    */
   public void validate();
   
   /**
    * Open a read-only buffer scope (for trigger support).
    */
   public void openReadOnlyScope();

   /**
    * Conversion of BUFFER-FIELD() method (KW_BUF_FLD).
    *
    * Gets the handle to a particular buffer of a query or ProDataSet object.
    *
    * @param   fieldNumber
    *          field number. An INTEGER expression representing the sequence
    *          number of the field in the buffer.
    *
    * @return  handle to a particular field in the buffer.
    */
   public handle bufferField(int fieldNumber);

   /**
    * Conversion of BUFFER-FIELD() method (KW_BUF_FLD).
    *
    * Gets the handle to a particular buffer of a query or ProDataSet object.
    *
    * @param   fieldName
    *          field name. A CHARACTER string expression representing the name
    *          of the field in the buffer.
    *                                                     bufferName
    * @return  handle to a particular field in the buffer.
    */
   public handle bufferField(String fieldName);

   /**
    * Conversion of BUFFER-COPY() method (KW_BUF_COPY).
    *
    * This method copies any common fields, determined by name, data type,
    * and extent-matching, from the source buffer to the receiving buffer.
    * If there are fields in one buffer that do not exist in the other, they
    * are ignored. This method is used to accommodate temp-tables of joins.
    *
    * @return  <code>true</code> if BUFFER-COPY() succeeds,
    *          else <code>false</code>.
    */
   public logical bufferCopy(handle bufHandle);

   /**
    * Conversion of FIND-FIRST() method (KW_FIND_1ST).
    *
    * Gets a single record. This method lets a user get the last record that
    * satisfies the predicate expression.
    *
    * @param   whereClause
    *          where clause, predicate-expression.
    *
    * @return  <code>true</code> if FIND-FIRST() succeeds,
    *          else <code>false</code>.
    */
   public logical findFirst(String whereClause);

   /**
    * Conversion of FIND-FIRST() method (KW_FIND_1ST).
    *
    * Gets a single record. This method lets a user get the last record that
    * satisfies the predicate expression.
    *
    * @param   whereClause
    *          where clause, predicate-expression.
    * @param   lock
    *          SHARE-LOCK|EXCLUSIVE-LOCK|NO-LOCK
    *
    * @return  <code>true</code> if FIND-FIRST() succeeds,
    *          else <code>false</code>.
    */
   public logical findFirst(String whereClause, int lock);

   /**
    * Conversion of FIND-FIRST() method (KW_FIND_1ST).
    *
    * Gets a single record. This method lets a user get the last record that
    * satisfies the predicate expression.
    *
    * @param   whereClause
    *          where clause, predicate-expression.
    * @param   lock
    *          SHARE-LOCK|EXCLUSIVE-LOCK|NO-LOCK
    *
    * @return  <code>true</code> if FIND-FIRST() succeeds,
    *          else <code>false</code>.
    */
   public logical findFirst(String whereClause, integer lock);

   /**
    * Conversion of FIND-FIRST() method (KW_FIND_1ST).
    *
    * Gets a single record. This method lets a user get the last record that
    * satisfies the predicate expression.
    *
    * @param   whereClause
    *          where clause, predicate-expression.
    * @param   lock
    *          SHARE-LOCK|EXCLUSIVE-LOCK|NO-LOCK
    * @param   nowait
    *          NO-WAIT
    *
    * @return  <code>true</code> if FIND-FIRST() succeeds,
    *          else <code>false</code>.
    */
   public logical findFirst(String whereClause, int lock, int nowait);

   /**
    * Conversion of FIND-FIRST() method (KW_FIND_1ST).
    *
    * Gets a single record. This method lets a user get the last record that
    * satisfies the predicate expression.
    *
    * @param   whereClause
    *          where clause, predicate-expression.
    * @param   lock
    *          SHARE-LOCK|EXCLUSIVE-LOCK|NO-LOCK
    * @param   nowait
    *          NO-WAIT
    *
    * @return  <code>true</code> if FIND-FIRST() succeeds,
    *          else <code>false</code>.
    */
   public logical findFirst(String whereClause, integer lock, int nowait);

   /**
    * Conversion of FIND-FIRST() method (KW_FIND_1ST).
    *
    * Gets a single record. This method lets a user get the last record that
    * satisfies the predicate expression.
    *
    * @param   whereClause
    *          where clause, predicate-expression.
    * @param   lock
    *          SHARE-LOCK|EXCLUSIVE-LOCK|NO-LOCK
    * @param   nowait
    *          NO-WAIT
    *
    * @return  <code>true</code> if FIND-FIRST() succeeds,
    *          else <code>false</code>.
    */
   public logical findFirst(String whereClause, int lock, integer nowait);

   /**
    * Conversion of FIND-FIRST() method (KW_FIND_1ST).
    *
    * Gets a single record. This method lets a user get the last record that
    * satisfies the predicate expression.
    *
    * @param   whereClause
    *          where clause, predicate-expression.
    * @param   lock
    *          SHARE-LOCK|EXCLUSIVE-LOCK|NO-LOCK
    * @param   nowait
    *          NO-WAIT
    *
    * @return  <code>true</code> if FIND-FIRST() succeeds,
    *          else <code>false</code>.
    */
   public logical findFirst(String whereClause, integer lock, integer nowait);

   /**
    * Conversion of FIND-LAST() method (KW_FIND_LST).
    *
    * Gets a single record. This method lets a user get the last record that
    * satisfies the predicate expression.
    *
    * @param   whereClause
    *          where clause, predicate-expression.
    *
    * @return  <code>true</code> if FIND-LAST() succeeds,
    *          else <code>false</code>.
    */
   public logical findLast(String whereClause);

   /**
    * Conversion of FIND-LAST() method (KW_FIND_LST).
    *
    * Gets a single record. This method lets a user get the last record that
    * satisfies the predicate expression.
    *
    * @param   whereClause
    *          where clause, predicate-expression.
    * @param   lock
    *          SHARE-LOCK|EXCLUSIVE-LOCK|NO-LOCK
    *
    * @return  <code>true</code> if FIND-LAST() succeeds,
    *          else <code>false</code>.
    */
   public logical findLast(String whereClause, int lock);

   /**
    * Conversion of FIND-LAST() method (KW_FIND_LST).
    *
    * Gets a single record. This method lets a user get the last record that
    * satisfies the predicate expression.
    *
    * @param   whereClause
    *          where clause, predicate-expression.
    * @param   lock
    *          SHARE-LOCK|EXCLUSIVE-LOCK|NO-LOCK
    *
    * @return  <code>true</code> if FIND-LAST() succeeds,
    *          else <code>false</code>.
    */
   public logical findLast(String whereClause, integer lock);

   /**
    * Conversion of FIND-LAST() method (KW_FIND_LST).
    *
    * Gets a single record. This method lets a user get the last record that
    * satisfies the predicate expression.
    *
    * @param   whereClause
    *          where clause, predicate-expression.
    * @param   lock
    *          SHARE-LOCK|EXCLUSIVE-LOCK|NO-LOCK
    * @param   nowait
    *          NO-WAIT
    *
    * @return  <code>true</code> if FIND-LAST() succeeds,
    *          else <code>false</code>.
    */
   public logical findLast(String whereClause, int lock, int nowait);

   /**
    * Conversion of FIND-LAST() method (KW_FIND_LST).
    *
    * Gets a single record. This method lets a user get the last record that
    * satisfies the predicate expression.
    *
    * @param   whereClause
    *          where clause, predicate-expression.
    * @param   lock
    *          SHARE-LOCK|EXCLUSIVE-LOCK|NO-LOCK
    * @param   nowait
    *          NO-WAIT
    *
    * @return  <code>true</code> if FIND-LAST() succeeds,
    *          else <code>false</code>.
    */
   public logical findLast(String whereClause, int lock, integer nowait);

   /**
    * Conversion of FIND-LAST() method (KW_FIND_LST).
    *
    * Gets a single record. This method lets a user get the last record that
    * satisfies the predicate expression.
    *
    * @param   whereClause
    *          where clause, predicate-expression.
    * @param   lock
    *          SHARE-LOCK|EXCLUSIVE-LOCK|NO-LOCK
    * @param   nowait
    *          NO-WAIT
    *
    * @return  <code>true</code> if FIND-LAST() succeeds,
    *          else <code>false</code>.
    */
   public logical findLast(String whereClause, integer lock, int nowait);

   /**
    * Conversion of FIND-LAST() method (KW_FIND_LST).
    *
    * Gets a single record. This method lets a user get the last record that
    * satisfies the predicate expression.
    *
    * @param   whereClause
    *          where clause, predicate-expression.
    * @param   lock
    *          SHARE-LOCK|EXCLUSIVE-LOCK|NO-LOCK
    * @param   nowait
    *          NO-WAIT
    *
    * @return  <code>true</code> if FIND-LAST() succeeds,
    *          else <code>false</code>.
    */
   public logical findLast(String whereClause, integer lock, integer nowait);

   /**
    * Conversion of FIND-UNIQUE() method (KW_FIND_UNI).
    *
    * Gets a single record. This method lets a user get a unique record that
    * satisfies the predicate expression.
    *
    * @param   whereClause
    *          where clause, predicate-expression.
    *
    * @return  <code>true</code> if FIND-UNIQUE() succeeds,
    *          else <code>false</code>.
    */
   public logical findUnique(String whereClause);

   /**
    * Conversion of FIND-UNIQUE() method (KW_FIND_UNI).
    *
    * Gets a single record. This method lets a user get a unique record that
    * satisfies the predicate expression.
    *
    * @param   whereClause
    *          where clause, predicate-expression.
    * @param   lock
    *          SHARE-LOCK|EXCLUSIVE-LOCK|NO-LOCK
    *
    * @return  <code>true</code> if FIND-UNIQUE() succeeds,
    *          else <code>false</code>.
    */
   public logical findUnique(String whereClause, int lock);

   /**
    * Conversion of FIND-UNIQUE() method (KW_FIND_UNI).
    *
    * Gets a single record. This method lets a user get a unique record that
    * satisfies the predicate expression.
    *
    * @param   whereClause
    *          where clause, predicate-expression.
    * @param   lock
    *          SHARE-LOCK|EXCLUSIVE-LOCK|NO-LOCK
    *
    * @return  <code>true</code> if FIND-UNIQUE() succeeds,
    *          else <code>false</code>.
    */
   public logical findUnique(String whereClause, integer lock);

   /**
    * Conversion of FIND-UNIQUE() method (KW_FIND_UNI).
    *
    * Gets a single record. This method lets a user get a unique record that
    * satisfies the predicate expression.
    *
    * @param   whereClause
    *          where clause, predicate-expression.
    * @param   lock
    *          SHARE-LOCK|EXCLUSIVE-LOCK|NO-LOCK
    * @param   nowait
    *          NO-WAIT
    *
    * @return  <code>true</code> if FIND-UNIQUE() succeeds,
    *          else <code>false</code>.
    */
   public logical findUnique(String whereClause, int lock, int nowait);

   /**
    * Conversion of FIND-UNIQUE() method (KW_FIND_UNI).
    *
    * Gets a single record. This method lets a user get a unique record that
    * satisfies the predicate expression.
    *
    * @param   whereClause
    *          where clause, predicate-expression.
    * @param   lock
    *          SHARE-LOCK|EXCLUSIVE-LOCK|NO-LOCK
    * @param   nowait
    *          NO-WAIT
    *
    * @return  <code>true</code> if FIND-UNIQUE() succeeds,
    *          else <code>false</code>.
    */
   public logical findUnique(String whereClause, int lock, integer nowait);

   /**
    * Conversion of FIND-UNIQUE() method (KW_FIND_UNI).
    *
    * Gets a single record. This method lets a user get a unique record that
    * satisfies the predicate expression.
    *
    * @param   whereClause
    *          where clause, predicate-expression.
    * @param   lock
    *          SHARE-LOCK|EXCLUSIVE-LOCK|NO-LOCK
    * @param   nowait
    *          NO-WAIT
    *
    * @return  <code>true</code> if FIND-UNIQUE() succeeds,
    *          else <code>false</code>.
    */
   public logical findUnique(String whereClause, integer lock, int nowait);

   /**
    * Conversion of FIND-UNIQUE() method (KW_FIND_UNI).
    *
    * Gets a single record. This method lets a user get a unique record that
    * satisfies the predicate expression.
    *
    * @param   whereClause
    *          where clause, predicate-expression.
    * @param   lock
    *          SHARE-LOCK|EXCLUSIVE-LOCK|NO-LOCK
    * @param   nowait
    *          NO-WAIT
    *
    * @return  <code>true</code> if FIND-UNIQUE() succeeds,
    *          else <code>false</code>.
    */
   public logical findUnique(String whereClause, integer lock, integer nowait);

   /**
    * Conversion of BUFFER-RELEASE() method (KW_BUF_REL).
    *
    * Releases a record from a buffer object. The BUFFER-RELEASE method
    * corresponds to the RELEASE statement.
    *
    * @return  <code>true</code> if BUFFER-RELEASE() succeeds,
    *          else <code>false</code>.
    */
   public logical bufferRelease();

   /**
    * Conversion of FIND-BY-ROWID() method (KW_FIND_BR).
    *
    * Locates the record with the rowid you specify, then moves the record
    * into the buffer.
    *
    * @param   id
    *          An expression of type ROWID that represents the rowid of the
    *          desired record.
    *
    * @return  <code>true</code> if it finds the record, else
    *          <code>false</code>.
    */
   public logical findByRowID(rowid id);

   /**
    * Conversion of FIND-BY-ROWID() method (KW_FIND_BR).
    *
    * Locates the record with the rowid you specify, then moves the record
    * into the buffer.
    *
    * @param   id
    *          An expression of type ROWID that represents the rowid of the
    *          desired record.
    * @param   lock
    *          SHARE-LOCK|EXCLUSIVE-LOCK|NO-LOCK
    *
    * @return  <code>true</code> if it finds the record, else
    *          <code>false</code>.
    */
   public logical findByRowID(rowid id, int lock);

   /**
    * Conversion of FIND-BY-ROWID() method (KW_FIND_BR).
    *
    * Locates the record with the rowid you specify, then moves the record
    * into the buffer.
    *
    * @param   id
    *          An expression of type ROWID that represents the rowid of the
    *          desired record.
    * @param   lock
    *          SHARE-LOCK|EXCLUSIVE-LOCK|NO-LOCK
    *
    * @return  <code>true</code> if it finds the record, else
    *          <code>false</code>.
    */
   public logical findByRowID(rowid id, integer lock);

   /**
    * Conversion of FIND-BY-ROWID() method (KW_FIND_BR).
    *
    * Locates the record with the rowid you specify, then moves the record
    * into the buffer.
    *
    * @param   id
    *          An expression of type ROWID that represents the rowid of the
    *          desired record.
    * @param   lock
    *          SHARE-LOCK|EXCLUSIVE-LOCK|NO-LOCK
    * @param   nowait
    *          NO-WAIT
    *
    * @return  <code>true</code> if it finds the record, else
    *          <code>false</code>.
    */
   public logical findByRowID(rowid id, integer lock, integer nowait);

   /**
    * Conversion of FIND-BY-ROWID() method (KW_FIND_BR).
    *
    * Locates the record with the rowid you specify, then moves the record
    * into the buffer.
    *
    * @param   id
    *          An expression of type ROWID that represents the rowid of the
    *          desired record.
    * @param   lock
    *          SHARE-LOCK|EXCLUSIVE-LOCK|NO-LOCK
    * @param   nowait
    *          NO-WAIT
    *
    * @return  <code>true</code> if it finds the record, else
    *          <code>false</code>.
    */
   public logical findByRowID(rowid id, int lock, integer nowait);

   /**
    * Conversion of FIND-BY-ROWID() method (KW_FIND_BR).
    *
    * Locates the record with the rowid you specify, then moves the record
    * into the buffer.
    *
    * @param   id
    *          An expression of type ROWID that represents the rowid of the
    *          desired record.
    * @param   lock
    *          SHARE-LOCK|EXCLUSIVE-LOCK|NO-LOCK
    * @param   nowait
    *          NO-WAIT
    *
    * @return  <code>true</code> if it finds the record, else
    *          <code>false</code>.
    */
   public logical findByRowID(rowid id, int lock, int nowait);

   /**
    * Conversion of FIND-BY-ROWID() method (KW_FIND_BR).
    *
    * Locates the record with the rowid you specify, then moves the record
    * into the buffer.
    *
    * @param   id
    *          An expression of type ROWID that represents the rowid of the
    *          desired record.
    * @param   lock
    *          SHARE-LOCK|EXCLUSIVE-LOCK|NO-LOCK
    * @param   nowait
    *          NO-WAIT
    *
    * @return  <code>true</code> if it finds the record, else
    *          <code>false</code>.
    */
   public logical findByRowID(rowid id, integer lock, int nowait);

   /**
    * Conversion of BUFFER-COMPARE() method (KW_BUF_COMP).
    *
    * This method does a rough compare of any common fields, determined by
    * name, data type, and extent-matching, between the source buffer and the
    * target buffer. The resulting logical value is either TRUE or FALSE as
    * a whole. A single field that does not compare causes the entire buffer
    * to return FALSE. If there are fields in one buffer that do not exist in
    * the other, they are ignored.
    *
    * @param   srcBufHandle
    *          An expression that evaluates to a buffer handle.
    *
    * @return  <code>true</code> if BUFFER-COMPARE() succeeds,
    *          else <code>false</code>.
    */
   public logical bufferCompare(handle srcBufHandle);


   /**
    * Conversion of NUM-FIELDS attribute (KW_NUM_FLD).
    *
    * Getter of NUM-FIELDS attribute.
    *
    * @return  current value of NUM-FIELDS attribute.
    */
   public integer numFields();

   /**
    * Conversion of INITIAL attribute (KW_INIT).
    *
    * Getter of INITIAL attribute.
    *
    * @return  current value of INITIAL attribute.
    */
   public character initial();
}
